% DownwardsInclinedPlane.m: Perform simulations on a downward inclined
%                           plane, and then compares the results with a
%                           flat plane. The beam propagating along the
%                           inclined plane has the same relative directivty
%                           as the flat plane. The comparison between the
%                           fields are done in the height direction.

clear all
% Setting the parameters for the inclined plane:
beta =(55/(2*360))*(2*pi);
A = 10^(0.9);
frequency = 110*10^6;

xDiff = 1000; 
zDiff = -20; 

theta0 = abs(asin(zDiff/xDiff)); % The tilt of the beam

deltaX = 1; 
deltaZ = 1;
maxX = 3100; 
minZ = -maxX*sin(theta0); 
numPtsAbsoptionLayer = 150;
xDist = 3000; 

deltaZarr =[1];
antHeight = 3; 
deltaXvect = [1]; 

% The irregular terrain:
Xn = [0  maxX];
Zn = [0 minZ];

maxZcompare = 270; 

% Interpolating the surface:
[xVect,zSurfaceVect] = interpolate(Xn,Zn,frequency,...
    'linear',deltaX);
% Shifts the surface:
[zSurfaceNorm,truncationValue]=normalizeSurface(zSurfaceVect);

% Adjusting the antenna height:
antHeight = antHeight + zSurfaceNorm(1);
maxHinterest = 330 +antHeight;

simulationType = cell(4,1);

% Plot the surface:
surfacePlot = figure();
plot(xVect,zSurfaceNorm)
xlabel('Distance [m]');
ylabel('Surface height [m]');
titleFig = [...
    'DownwardsInclinedPlane_results/DownwardsInclinedPlane_surface.png'];
saveas(surfacePlot,titleFig,'png');

% Creating initial field:
[zVectFDM,HindexFDM] =createZvectAbsorptionLayer2(...
    maxHinterest,deltaZ,numPtsAbsoptionLayer);
initialFieldFDM = createInitialField(antHeight,theta0,beta,...
    zVectFDM,A, frequency,'gaussian1');
numZpoints = length(zVectFDM);

% Calculating the field over the downwards inclined plane:
tic
uValuesSplitStep = SSAirregularTerrainAbsoptionLayer(...
    initialFieldFDM,zVectFDM,...
    xVect,zSurfaceNorm,HindexFDM,frequency,...
    numPtsAbsoptionLayer,antHeight);
toc
tic
uValuesFDM = FDMirregularTerrainAbsorptionLayer ...
    (initialFieldFDM,zVectFDM,xVect,zSurfaceNorm,...
    HindexFDM,frequency,numPtsAbsoptionLayer);
toc
deltaZstr = num2str(deltaZ);
yText = strcat('Height above the lowest point [m]');

%maxZ = 250;

% Extracting the vertical values:
eFieldSSA  = verticalVector(getVerticalValues(...
    xVect,zSurfaceNorm,xDiff,zDiff,deltaX,deltaZ,...
    xVect,xDist,maxZcompare,uValuesSplitStep,'down'))';


eFieldFDM = verticalVector(getVerticalValues(...
    xVect,zSurfaceNorm,xDiff,zDiff,deltaX,deltaZ,...
    xVect,xDist,maxZcompare,uValuesFDM,'down'))';

simulationType{1,1} = ['SSA inclined'];

simulationType{2,1} = ['FDM inclined'];
tx = antHeight;
rx = antHeight;

eFieldTot = eFieldSSA;
eFieldTot = vertcat(eFieldTot,eFieldFDM);



minPlotLevel = 10^(-4);
for i = 1:1
% Plot SSA figure:
plotScale =length(zVectFDM); 
fig2 = figure('visible','off');
part1Title = ['Split-Step Algorithm - flat surface'];
titleVal2 = {part1Title};
tic
uValuesAux = uValuesSplitStep(1:plotScale,:);
uValuesAux(abs(uValuesAux)<minPlotLevel) = minPlotLevel;
minVal = 10.*log10(min(min(abs(uValuesAux).^2)));
maxVal = 10.*log10(max(max(abs(uValuesAux).^2)));
disp('contourf is on')
contourf(xVect,zVectFDM(1:plotScale,1),...
    10.*log10(abs(uValuesAux.^2)),50)
hold on
disp('contour is on')
contour(xVect,zVectFDM(1:plotScale,1),...
    10.*log10(abs(uValuesAux.^2)),50)
hold on
eFieldSSA  = verticalVector(getVerticalValues(xVect,...
    zSurfaceNorm,xDiff,zDiff,deltaX,deltaZ,...
    xVect,xDist,maxZcompare,uValuesSplitStep,'down'))';

%title(titleVal2)
xlabel('Distance [m]');
ylabel(yText);
grid on

titleFig = ['DownwardsInclinedPlane_results/SSA_InclDown.png'];
disp('saving ...')
saveas(fig2,titleFig,'png');
toc

tic
% Plot FDM figure:
fig = figure('visible','off');
part1Title = ['Finite-Difference Method - flat surface'];
titleVal2 = {part1Title};

uValuesAux = uValuesFDM(1:plotScale,:);
uValuesAux(abs(uValuesAux)<minPlotLevel) = minPlotLevel;
minVal = 10.*log10(min(min(abs(uValuesAux).^2)));
maxVal = 10.*log10(max(max(abs(uValuesAux).^2)));
contourf(xVect,zVectFDM(1:plotScale,1),...
    10.*log10(abs(uValuesAux.^2)),50)
hold on
contour(xVect,zVectFDM(1:plotScale,1),...
    10.*log10(abs(uValuesAux.^2)),50)
hold on
eFieldFDM = verticalVector(getVerticalValues(xVect,...
    zSurfaceNorm,xDiff,zDiff,deltaX,deltaZ,...
    xVect,xDist,maxZcompare,uValuesFDM,'down'))';
%title(titleVal2)
xlabel('Distance [m]');
ylabel(yText);
grid on
titleFig = ['DownwardsInclinedPlane_results/FDM_InclDown.png'];
saveas(fig,titleFig,'png');
toc

end % Plot figure, not in use

% Comparing with results from Indra (along the surface),
% horizontal comparison:

tx =antHeight;
rx = antHeight;
rx_xArr = xVect;
rx_zArr = ones(length(xVect),1).*antHeight;

plotTitle = ' '; 
titleFig = ['DownwardsInclinedPlane_results/SSA_FDM_InclDown.png'];
filename = 'IndraSource/wedge1/LPDA-u-kile_E.xls';


rx_xArr = ones(length(zVectFDM),1).*length(xVect);
rx_zArr = zVectFDM;

simulationType1 = cell(2,1);
simulationType1{1,1} = 'SSA inclined';
simulationType1{2,1} = 'FDM inclined';

pathLossWedge_Indra(Xn,Zn,deltaX,A,tx,rx_xArr,rx_zArr,eFieldTot,...
    antHeight,beta,frequency,simulationType1, plotTitle,titleFig,filename)
%zDist = 250;


% Calculating the field over a flat surface
antHeight = antHeight - zSurfaceNorm(1);
theta0 = 0;
%maxHeight = 250;
xVect = verticalVector([0:deltaX:xDist]);
% Creating initial field:
[zVectFDM,HindexFDM] =createZvectAbsorptionLayer2(...
    maxHinterest,deltaZ,numPtsAbsoptionLayer);
initialFieldFDM = createInitialField(antHeight,theta0,beta,...
    zVectFDM,A, frequency,'gaussian1');
numZpoints = length(zVectFDM);

% Calculating field:
tic
uValuesSplitStep_flat =splitStepAlgorithmAbsorptionLayer(...
    initialFieldFDM,zVectFDM,xVect, ...
    HindexFDM,frequency,numPtsAbsoptionLayer);
toc
tic
[uValuesFDMalne_flat,maxEigVal]=...
    FDMAbsorptionLayerNumEfficient2(initialFieldFDM, ...
    zVectFDM,xVect,HindexFDM,frequency,numPtsAbsoptionLayer);
toc

% Extracting the results for vertical comparison:
eFieldSSA = zeros(1,maxZcompare+1); %maxHeight+1);
eFieldFDM = zeros(1,maxZcompare+1); %maxHeight+1);
for i = 1:maxZcompare+1 %maxHeight+1
    eFieldSSA(1,i) = uValuesSplitStep_flat(i,...
        length(xVect));
    eFieldFDM(1,i) = uValuesFDMalne_flat(i,...
        length(xVect));
end

simulationType{3,1} = ['SSA flat'];
simulationType{4,1} = ['FDM flat'];
eFieldTot = vertcat(eFieldTot,eFieldSSA);
eFieldTot = vertcat(eFieldTot,eFieldFDM);

% Comparing with results from Indra (along the surface), 
% horizontal comparison:
tx =antHeight;
rx = antHeight;
rx_xArr = ones(length(zVectFDM),1).*length(xVect);
rx_zArr = zVectFDM;
part1Title = ' ';%['SSA and FDM - flat surface - along the surface'];
plotTitle = ' '; % {part1Title};

titleFig_comp = [...
    'DownwardsInclinedPlane_results/SSA_FDM_compare_DownIncl.png'];
filename = 'IndraWedge_results/LPDA-u-kile_E.xls';
numCases = 4;
startIndex = 4;
numElts = length(zVectFDM);

eFieldTot1 = zeros(numCases,length(eFieldTot(1,:))); %startIndex:numElts)));
for i = 1:numCases
   eFieldTot1(i,:) = eFieldTot(i,:); %startIndex:numElts);
end


compareHeight = 50;
 titleFig3 =['DownwardsInclinedPlane_results/',...
     'SSA_FDM_flat_height_varying_DownIncl_zoomed1.png']; 
  pathLossFlat_Indra_minComp(rx_xArr,rx_zArr,eFieldTot1,...
     frequency,simulationType, plotTitle,titleFig3,filename,compareHeight)
 
 titleFig4 =['DownwardsInclinedPlane_results/',...
     'SSA_FDM_flat_height_varying_DownIncl_zoomed2.png'];
  pathLossFlat_Indra(rx_xArr,rx_zArr,eFieldTot1,...
     frequency,simulationType, plotTitle,titleFig4,filename,compareHeight)

compareHeight = 240;
 
 pathLossFlat_Indra(rx_xArr,rx_zArr,eFieldTot1,...
   frequency,simulationType, plotTitle,titleFig_comp,filename,compareHeight)
 
% Plot SSA figure:
yText = strcat('Height above surface [m]');
fig2 = figure('visible','off');
part1Title = ['Split-Step Algorithm - flat surface'];
titleVal2 = {part1Title};

uValuesAux = uValuesSplitStep_flat;
uValuesAux(abs(uValuesAux)<10^-11) = 10^-11;
contourf(xVect,zVectFDM,10.*log10(abs(uValuesAux.^2)),50)
hold on
contour(xVect,zVectFDM,10.*log10(abs(uValuesAux.^2)),50)
%title(titleVal2)
xlabel('Distance [m]');
ylabel(yText);
grid on
titleFig = ['DownwardsInclinedPlane_results/SSA_flat_InclDown.png'];
saveas(fig2,titleFig,'png');

% Plot FDM figure:
fig = figure('visible','off');
part1Title = ['Finite-Difference Method - flat surface'];
titleVal2 = {part1Title};

uValuesAux = uValuesFDMalne_flat;
uValuesAux(abs(uValuesAux)<10^-11) = 10^-11;
contourf(xVect,zVectFDM,10.*log10(abs(uValuesAux.^2)),50)
hold on
contour(xVect,zVectFDM,10.*log10(abs(uValuesAux.^2)),50)
%title(titleVal2)
xlabel('Distance [m]');
ylabel(yText);
grid on
titleFig = ['DownwardsInclinedPlane_results/FDM_flat_InclDown.png'];
saveas(fig,titleFig,'png');


% Compare along the surface at constant height of 40 m above the lowest
% point:

%eFieldAlongTot 
height = 40;
startIndex = 101; %100;
eSSArunway = (uValuesSplitStep(height,startIndex:length(xVect)));
eFieldAlongTot  =  eSSArunway;
eFDMrunway = (uValuesFDM(height,startIndex:length(xVect)));
eFieldAlongTot = vertcat(eFieldAlongTot,eFDMrunway);

eSSAflat = (uValuesSplitStep_flat(height,startIndex:length(xVect)));
eFieldAlongTot = vertcat(eFieldAlongTot,eSSAflat);
eFDMflat = (uValuesFDMalne_flat(height,startIndex:length(xVect)));
eFieldAlongTot = vertcat(eFieldAlongTot,eFDMflat);

imulationType2 = cell(4,1);
simulationType2{1,1} = ['SSA - downwards'];
simulationType2{2,1} = ['FDM - downwards'];
simulationType2{3,1} = ['SSA - flat'];
simulationType2{4,1} = ['FDM - flat'];

rx_xArr3 = xVect(startIndex:length(xVect));
rx_zArr3 =  ones(length(rx_xArr3),1).*height;
titleFig3 = ['DownwardsInclinedPlane_results/',...
    'SSA_FDM_pathloss_horizontal_InclDown.png'];
antHeight2 = antHeight - zSurfaceNorm(1);

titleFig4 = ['DownwardsInclinedPlane_results/',...
    'SSA_FDM_pathloss_horizontal2_InclDown.png'];

pathLossIndra_alongX(rx_xArr3,rx_zArr3,eFieldAlongTot,...
    frequency,simulationType2,' ',titleFig4,' ')


% Comparing fields along the surface at constant height above the surface
height2 = 15;
startIndex2 = 101;

fieldVect2 = [ceil(startIndex2/deltaX):length(xVect)/deltaX];
eSSArunway2 = zeros(length(fieldVect2),1);
eFDMrunway2 = zeros(length(fieldVect2),1);
for i = 1:length(fieldVect2)
    eSSArunway2(i,1) = uValuesSplitStep(round(zSurfaceNorm(i)+height), ...
        fieldVect2(i));
    eFDMrunway2(i,1) = uValuesFDM(round(zSurfaceNorm(i)+height), ...
        fieldVect2(i));
end

eFieldAlongTot2  =  eSSArunway2';
eFieldAlongTot2 = vertcat(eFieldAlongTot2,eFDMrunway2');
eSSAflat2 = (uValuesSplitStep_flat(height,startIndex2:length(xVect)));
eFieldAlongTot2 = vertcat(eFieldAlongTot2,eSSAflat2);
eFDMflat2 = (uValuesFDMalne_flat(height,startIndex2:length(xVect)));
eFieldAlongTot2 = vertcat(eFieldAlongTot2,eFDMflat2);

titleFig5 = ['DownwardsInclinedPlane_results/',...
    'SSA_FDM_pathloss_horizontal_cst_diff_surface_InclDown.png'];
rx_xArr4 = xVect(startIndex2:length(xVect));
rx_zArr4 =  ones(length(rx_xArr4),1).*height;


pathLossIndra_alongX(rx_xArr4,rx_zArr4,eFieldAlongTot2,...
    frequency,simulationType2,' ',titleFig5,' ')

% Plotting the surface profile with logaritmic axes:
fig = figure();    
semilogx((xVect(startIndex2:length(xVect))),zSurfaceNorm...
     (startIndex2:length(xVect))); %,'Parent',ax2);

legend('Surface profile','Location','SouthWest');
xlabel('Distance [m]');
ylabel('Height [m]');
plotwidth = 560;
plotheight = 200;
set(fig, 'Position', [500 100 plotwidth plotheight]);
grid on
titleFig6 = ['DownwardsInclinedPlane_results/',...
    'Runway_profile_small_InclDown.png'];
saveas(fig,titleFig,'png');
titleFig6 = titleFig6(1:(length(titleFig6)-4));
titleFig6 = horzcat(titleFig6,'.pdf');
%print (fig, '-dpdf', titleFig);
save2pdf(titleFig6);




