
% finiteDifferenceMethodAbsorptionLayer.m : Calculating the electric field 
%                                           using the
%                                           finite-difference method.
% Assumption: Propagation in vacuum, no variations in the refractive index,
%             n(x,z) = 1 up to the max height of consideration, for all x and z.

% zFieldInit: Vector containing the initial electric field along the z-axis
% zVect: Vector containing the z-coordinates of interest
% xVect: Vector containing the x-coordinates of interest
% maxHeigthInterestZIndex: the z-index that contains the highest index of
%                          interest, above this index there will be 
%                          absorption in order to avoid reflection from the
%                          sky.
% frequency: The frequency of operation

function [uValues,maxEigVal] = FDMAbsorptionLayerNumEfficient(zFieldInit, ...
    zVect,xVect,maxHeigthInterestZIndex,frequency,g)
c = 3*10^8;
lambda = c/frequency;
k = 2*pi/lambda;
numZpoints = length(zVect);
numXpoints = length(xVect);
Hindex = maxHeigthInterestZIndex;
xVect= verticalVector(xVect);
zVect = verticalVector(zVect);

deltaZ = zVect(2,1)-zVect(1,1);
deltaZvect = zeros(numZpoints,1);
deltaZvect(1:numZpoints-1,1)= zVect(2:numZpoints,1)-zVect(1:numZpoints-1);
deltaZvect(numZpoints,1) = deltaZvect(numZpoints-1,1);

deltaX = xVect(2,1)-xVect(1,1);
deltaXvect = zeros(numXpoints,1);
deltaXvect(1:numXpoints-1,1)= xVect(2:numXpoints,1)-xVect(1:numXpoints-1);
deltaXvect(numXpoints,1) = deltaXvect(numXpoints-1,1);

% Creating absorption layer:
indiceValues = verticalVector([1:1:(numZpoints-Hindex+1)]);
numAbsLpoints = numZpoints-Hindex+1; % Number of points in the absorption layer
numInt = 20;
numEltsGroup = ceil(numAbsLpoints./numInt);
indiceValues = ceil(indiceValues./numEltsGroup);
% Hanning window for absorption layer:
%indiceValues = (verticalVector([0:1:numZpoints-Hindex]))./(numZpoints-Hindex);
%absorptionLayer = (1 +cos(pi.*indiceValues))./2;


% Creating the absortion layer using a damping function, geometrical
% damping
gamma0 = zFieldInit(Hindex,1); % 0.1;
%g = 0.01;
absorptionLayer = gamma0.*(g.^(indiceValues./deltaZvect(1,1)));

indexRefraction = ones(numZpoints,1);
indexRefraction(Hindex:numZpoints,1) = indexRefraction(Hindex:numZpoints,1) ...
    + 1j.*absorptionLayer;

% Finding the ''grid'' of a- and b-values:



% Vector a- and b-grid:
%bGrid = zeros(numZpoints,1);
aGrid = zeros(numZpoints,1);
%bGrid(:,1) = 1j.*4.*k.*(deltaZvect.^2)./deltaX;
bGrid = 1j.*4.*k.*(deltaZ.^2)./deltaX;
aGrid(:,1) = k.^2.*(indexRefraction.^2 -1).*(deltaZ.^2);


% Calculate the u-values for successive x-values:
uGrid = zeros(numZpoints,numXpoints);
uGrid(:,1)= zFieldInit;
onesVect = ones(numZpoints-1,1);
%VmMat = zeros(numZpoints,numZpoints);
%VmRes = zeros(numZpoints,1);
%AmMat =  zeros(numZpoints,numZpoints);

%disp('Create Vmat')
diagVmat = 2 + bGrid(1,1)- aGrid(:,1);
diagVmat(1,1) = 1;
diagVmat(numZpoints,1) = 1;
VmMat = (diag(diagVmat)+ (-1.*diag(onesVect,-1))+(-1.*diag(onesVect,1))); 
VmMat(1,2) = 0;
VmMat(numZpoints,numZpoints-1) = 0;
%disp('Create Amat')
diagAmat = -2 + bGrid(1,1)+ aGrid(:,1);
diagAmat(1,1) = 1;
diagAmat(numZpoints,1) = 1;
AmMat = (diag(diagAmat)+diag(onesVect,-1) + diag(onesVect,1));
AmMat(1,2) = 0;
AmMat(numZpoints,numZpoints-1) = 0;

%disp('Create transition matrix')
transitionMat = (VmMat/AmMat);

% Diagonalization of the transition matrix:
%disp('Eigenvalues start');
%tic
[eigVect,eigValues] = eig(transitionMat);
%toc
%disp('Eigenvalues finished');
maxEigVal=max(max(abs(eigValues)))

% Check if the transition matrix can be inverted (check if all the
% eigenvectors are linearly independent, same as full rank transition 
% matrix):
%if(max(max(abs(eigValues))) >= 1)
 %   error('FDMAbsorptionLayerNumEfficient:The absolute value of the largest eigenvalue >=1 -> Divergence',...
 %       'The absolute value of the largest eigenvalue >=1 -> Divergence');
%end
%disp('Start inverse')
eigVectInv = sparse(pinv(eigVect));
%disp('Inversed')
eigVect = sparse(eigVect);
%numEigValues = length(eigValues);
eigValues = sparse(eigValues);

initEigInv = sparse(eigVectInv*zFieldInit);


auxEigValues = eigValues;
parfor x = 2:numXpoints
    %x
    %tic
    %auxEigValues = auxEigValues.*eigValues;
    auxEigValues = eigValues.^x;
    uGrid(:,x) = eigVect*auxEigValues*initEigInv;
    %toc
end

uValues = uGrid;
%uValues(abs(uValues)<10^-3) = 10^-3; 
%uValues(abs(uValues)>10^-1) = 10^-1;
end
