
% FDMAbsorptionLayerNumEfficient2.m : Calculating the electric field over a
%                                     flat surface using the 
%                                     finite-difference method.     

% zFieldInit: Vector containing the initial electric field along the z-axis
% zVect: Vector containing the z-coordinates of interest
% xVect: Vector containing the x-coordinates of interest
% maxHeigthInterestZIndex: the z-index that contains the highest index of
%                          interest, above this index there will be 
%                          absorption in order to avoid reflection from the
%                          sky.
% frequency: The frequency of operation
% numPointsInAbsLayer: The thickness of the absorption layer in the number
%                      of points.

% return: uValues: The simulated field.
%         maxEigVal: The maximum eigenvalue of the transistion matrix,
%                    gives the stability of the system.

function [uValues,maxEigVal] = FDMAbsorptionLayerNumEfficient2(zFieldInit, ...
    zVect,xVect,maxHeigthInterestZIndex,frequency,numPointsInAbsLayer)
c = 3*10^8;
lambda = c/frequency;
k = 2*pi/lambda;
numZpoints = length(zVect);
numXpoints = length(xVect);
Hindex = maxHeigthInterestZIndex;
xVect= verticalVector(xVect);
zVect = verticalVector(zVect);

deltaZ = zVect(2,1)-zVect(1,1);
deltaZvect = zeros(numZpoints,1);
deltaZvect(1:numZpoints-1,1)= zVect(2:numZpoints,1)-zVect(1:numZpoints-1);
deltaZvect(numZpoints,1) = deltaZvect(numZpoints-1,1);

deltaX = xVect(2,1)-xVect(1,1);
deltaXvect = zeros(numXpoints,1);
deltaXvect(1:numXpoints-1,1)= xVect(2:numXpoints,1)-xVect(1:numXpoints-1);
deltaXvect(numXpoints,1) = deltaXvect(numXpoints-1,1);

% Creating absorption layer:
indexRefraction = createAbsorptionLayer(zFieldInit,...
    Hindex,numPointsInAbsLayer);

% Finding the ''grid'' of a- and b-values:
% Vector a- and b-grid:

aGrid = zeros(numZpoints,1);
bGrid = 1j.*4.*k.*(deltaZ.^2)./deltaX;
aGrid(:,1) = k.^2.*(indexRefraction.^2 -1).*(deltaZ.^2);


% Creating grid for the uValues with the initial field:
uGrid = zeros(numZpoints,numXpoints);
uGrid(:,1)= zFieldInit;
onesVect = ones(numZpoints-1,1);

% Create Vmat: 
diagVmat = 2 + bGrid(1,1)- aGrid(:,1);
diagVmat(1,1) = 1;
diagVmat(numZpoints,1) = 1;
VmMat = (diag(diagVmat)+ (-1.*diag(onesVect,-1))+(-1.*diag(onesVect,1))); 
VmMat(1,2) = 0;
VmMat(numZpoints,numZpoints-1) = 0;

% Create Amat: 
diagAmat = -2 + bGrid(1,1)+ aGrid(:,1);
diagAmat(1,1) = 1;
diagAmat(numZpoints,1) = 1;
AmMat = (diag(diagAmat)+diag(onesVect,-1) + diag(onesVect,1));
AmMat(1,2) = 0;
AmMat(numZpoints,numZpoints-1) = 0;

% Creating transition marix:
transitionMat = (VmMat/AmMat);

% Diagonalization of the transition matrix:
[eigVect,eigValues] = eig(transitionMat);
maxEigVal=max(max(abs(eigValues)))

% Taking the pseudo-inverse of eigVect:
eigVectInv = sparse(pinv(eigVect));
eigVect = sparse(eigVect);
eigValues = sparse(eigValues);

initEigInv = sparse(eigVectInv*zFieldInit);

auxEigValues = eigValues;
% Calculating the field:
parfor x = 2:numXpoints
    auxEigValues = eigValues.^x;
    uGrid(:,x) = eigVect*auxEigValues*initEigInv;
end

uValues = uGrid;
end
