% FDMirregularTerrainAbsorptionLayer.m : Calculates the electric field 
%                            using the finite difference method for a given
%                            terrain profile. The function
%                            has an absorbing layer at the top, preventing
%                            the simulations to have reflections from the
%                            sky.

% zFieldInit: Vector containing the initial electric field along the z-axis
% zVect: Vector containing the z-coordinates of interest
% xVect: Vector containing the x-coordinates of interest
% zSurfaceVect: Vector containing the z-coordinates of the surface
% maxHeigthInterestZIndex: the z-index that contains the highest index of
%                          interest, above this index there will be 
%                          absorption in order to avoid reflection from the
%                          sky.
% frequency: The frequency of operation
% numPointsInAborptionLayer: Number of points in the absorption layer.

% return: uValues: The calculated u-values.

function [uValues] = FDMirregularTerrainAbsorptionLayer(zFieldInit, ...
    zVect,xVect,zSurfaceVect,maxHeigthInterestZIndex,frequency, ...
    numPointsInAborptionLayer)

c = 3*10^8;
lambda = c/frequency;
k = 2*pi/lambda;
numZpoints = length(zVect);
numXpoints = length(xVect);
zSurfaceVect = verticalVector(zSurfaceVect);
xVect = verticalVector(xVect);
zVect= verticalVector(zVect);
Hindex = maxHeigthInterestZIndex;

% Mapping the surface-vector to correspond with the heights in zVect
% The values of the new surface vector contains the indices corresponding
% to the height in the zVect
zSurfaceIndices = zeros(numXpoints,1);
for x = 1:numXpoints
    zSurfaceIndices(x,1) = find(zVect >=zSurfaceVect(x,1),1,'first');
end

% Hanning window for absorption layer:
% indiceValues = (verticalVector([0:1:numZpoints-Hindex]))./(numZpoints-Hindex);
% absorptionLayer = (1 +cos(pi.*indiceValues))./2;
% indexRefraction = ones(numZpoints,1);
% 
% indiceValues = verticalVector([1:1:(numZpoints-Hindex+1)]);
% indiceValues = (indiceValues./(numPointsPerLayer*numLayers));
indexRefraction = createAbsorptionLayer(zFieldInit,...
    Hindex,numPointsInAborptionLayer);
%indexRefraction(Hindex:numZpoints,1) = indexRefraction(Hindex:numZpoints,1) ...
%    + 1j.*absorptionLayer;

% Finding the ''grid'' of a- and b-values:
deltaZ = zVect(2,1)-zVect(1,1);
deltaX = xVect(2,1)-xVect(1,1);
bGrid = 1j.*ones(numZpoints,1).*4.*k.*(deltaZ.^2)./deltaX;
aGrid = ones(numZpoints,1).*k.^2.*(indexRefraction.^2 -1).*deltaZ.^2;


% Calculate the u-values for successive x-values:
uGrid = zeros(numZpoints,numXpoints);
uGrid(:,1)= zFieldInit;
onesVect = ones(numZpoints-1,1);
VmMat = zeros(numZpoints,numZpoints);
VmRes = zeros(numZpoints,1);
AmMat =  zeros(numZpoints,numZpoints);

 diagVmat = 2 + bGrid(:,1)- aGrid(:,1);
    diagVmat(1,1) = 1;
    diagVmat(numZpoints,1) = 1;
    VmMat = diag(diagVmat)+ (-1.*diag(onesVect,-1))+(-1.*diag(onesVect,1)); 
    VmMat(1,2) = 0;
    VmMat(numZpoints,numZpoints-1) = 0;
    
     diagAmat = -2 + bGrid(:,1)+ aGrid(:,1);
    diagAmat(1,1) = 1;
    diagAmat(numZpoints,1) = 1;
     AmMat = diag(diagAmat)+diag(onesVect,-1) + diag(onesVect,1);
     AmMat(1,2)= 0;
     AmMat(numZpoints,numZpoints-1) = 0;

for x = 2:numXpoints
    % Filling in the Um and Vm matrices for field propagation estimation:
   
    VmRes = VmMat*uGrid(:,x-1);
    
   
   
    %AmMat*uGrid(:,x) = VmRes
    uGrid(:,x) = linsolve(AmMat,VmRes);
    
    if zSurfaceIndices(x,1) > 1
        uGrid(1:(zSurfaceIndices(x,1)-1),x) = 0;
        %uGrid(1:(zSurfaceIndices(x,1)),x) = 0;
    end
    %x
        
end

uValues = uGrid;
%uValues(abs(uValues)<10^-3) = 10^-3; 
%uValues(abs(uValues)>10^-1) = 10^-1;
end
