
% FDMnoGrond.m : Calculating the electric field in free-space
%                using the finite-difference method.
%                                         
% Assumption: Propagation in vacuum, no variations in the refractive index,
%             n(x,z) = 1 up to the max height of consideration, for all x 
%             and z.

% zFieldInit: Vector containing the initial electric field along the z-axis
% zVect: Vector containing the z-coordinates of interest
% xVect: Vector containing the x-coordinates of interest
% maxHeigthInterestZIndex: the z-index that contains the highest index of
%                          interest, above this index there will be 
%                          absorption in order to avoid reflection from the
%                          sky.
% frequency: The frequency of operation
% antennaIndex: The index of the center source point of the field in the
%               initital field.
%
% return: uValues: The u-values (electric field) for the entire
%                  computational domain.
%         maxEigVal: The maximum eigenvalue of the ''system'' matrix that
%                    is raisd to the n'th power.
%         antennaSourceIndex: The height index of the center point of the
%                             source for the output field.

function [uValues,maxEigVal,antennaSourceIndex] = FDMnoGround(zFieldInit, ...
    zVect,xVect,maxHeigthInterestZIndex,frequency,numPointsInAbsLayer,...
    antennaIndex)
c = 3*10^8;
lambda = c/frequency;
k = 2*pi/lambda;
numZpoints = length(zVect);
numXpoints = length(xVect);
Hindex = maxHeigthInterestZIndex;
xVect= verticalVector(xVect);
zVect = verticalVector(zVect);

deltaZ = zVect(2,1)-zVect(1,1);
deltaZvect = zeros(numZpoints,1);
deltaZvect(1:numZpoints-1,1)= zVect(2:numZpoints,1)-zVect(1:numZpoints-1);
deltaZvect(numZpoints,1) = deltaZvect(numZpoints-1,1);

deltaX = xVect(2,1)-xVect(1,1);
deltaXvect = zeros(numXpoints,1);
deltaXvect(1:numXpoints-1,1)= xVect(2:numXpoints,1)-xVect(1:numXpoints-1);
deltaXvect(numXpoints,1) = deltaXvect(numXpoints-1,1);

% Flipping the Gaussian beam around the source center:
noGroundVect = zFieldInit(antennaIndex:numZpoints,1);
auxVect = flipud(noGroundVect(2:length(noGroundVect),1));
antennaSourceIndex = length(auxVect) +1;



% Creating absorption layer:
indexRefraction = createAbsorptionLayer(zFieldInit,...
    Hindex,numPointsInAbsLayer);
indexRefraction = verticalVector(indexRefraction);

auxRefraction = indexRefraction(antennaIndex:numZpoints,1);
auxRefraction2 = flipud(auxRefraction(2:length(auxRefraction),1)); 
indexRefraction = vertcat(auxRefraction2,auxRefraction);

zFieldInit = vertcat(auxVect,noGroundVect);
numZpoints = length(indexRefraction);



% Finding the ''grid'' of a- and b-values:

% Vector a- and b-grid:

aGrid = zeros(numZpoints,1);
bGrid = 1j.*4.*k.*(deltaZ.^2)./deltaX;
aGrid(:,1) = k.^2.*(indexRefraction.^2 -1).*(deltaZ.^2);


% Calculate the u-values for successive x-values:
uGrid = zeros(numZpoints,numXpoints);
uGrid(:,1)= zFieldInit;
onesVect = ones(numZpoints-1,1);

% Creates VmMat:
diagVmat = 2 + bGrid(1,1)- aGrid(:,1);
diagVmat(1,1) = 1;
diagVmat(numZpoints,1) = 1;
VmMat = (diag(diagVmat)+ (-1.*diag(onesVect,-1))+(-1.*diag(onesVect,1))); 
VmMat(1,2) = 0;
VmMat(numZpoints,numZpoints-1) = 0;

% Creates Amat: 
diagAmat = -2 + bGrid(1,1)+ aGrid(:,1);
diagAmat(1,1) = 1;
diagAmat(numZpoints,1) = 1;
AmMat = (diag(diagAmat)+diag(onesVect,-1) + diag(onesVect,1));
AmMat(1,2) = 0;
AmMat(numZpoints,numZpoints-1) = 0;

% Create transition matrix:
transitionMat = (VmMat/AmMat);

% Diagonalization of the transition matrix:
%disp('Eigenvalues start');
%tic
[eigVect,eigValues] = eig(transitionMat);
%toc

maxEigVal=max(max(abs(eigValues)))

% Inversing eigVect using pseudo inverse:
eigVectInv = sparse(pinv(eigVect));
eigVect = sparse(eigVect);
eigValues = sparse(eigValues);

initEigInv = sparse(eigVectInv*zFieldInit);


auxEigValues = eigValues;
parfor x = 2:numXpoints
    auxEigValues = eigValues.^x;
    uGrid(:,x) = eigVect*auxEigValues*initEigInv;
  
end

uValues = uGrid;
end
