% splitStepAlgorithm.m: Calculates the electric field using the split-step
%                       algorithm
% Assumption: - Propagation in vacuum, no variations in the refractive index,
%               n(x,z) = 1, for all x and z.
% Remarks: length(xVect) >= length(zVect)             

% zFieldInit: Vector containing the initial electric field along the z-axis
% zVect: Vector containing the z-coordinates of interest
% xVect: Vector containing the x-coordinates of interest
% maxHeigthInterestZIndex: the z-index that contains the highest index of
%                          interest, above this index there will be 
%                          absorption in order to avoid reflection from the
%                          sky.
% frequency: The frequency of operation
% g: Parameter for the absorption layer
% alpha: Value used for boundary conditions, for horizontal polarization Re{alpha} < 0 (Levy, p.153)

function uValues = SSA_AbsorptionLayer_BC(zFieldInit,zVect, ...
    xVect,maxHeigthInterestZIndex,frequency,g,alpha)

c = 3*10^8;
lambda = c/frequency;
k = 2*pi/lambda;
numZpoints = length(zVect);
numXpoints = length(xVect);
Hindex = maxHeigthInterestZIndex;

deltaZvect = zeros(numZpoints,1);
deltaZvect(1:numZpoints-1,1)= zVect(2:numZpoints,1)-zVect(1:numZpoints-1);
deltaZvect(numZpoints,1) = deltaZvect(numZpoints-1,1);
deltaZ = deltaZvect(1,1);

deltaXvect = zeros(numXpoints,1);
deltaXvect(1:numXpoints-1,1)= xVect(2:numXpoints,1)-xVect(1:numXpoints-1);
deltaXvect(numXpoints,1) = deltaXvect(numXpoints-1,1);
deltaX = deltaXvect(1,1);

uGrid = zeros(numZpoints,numXpoints);
uGrid(:,1)= zFieldInit;
indicesZminusOne = verticalVector(linspace(1,(numZpoints-1),(numZpoints-1)));
Pprime = exp(-1j.*(pi.^2).*(indicesZminusOne.^2)...
    .*deltaXvect(1:1,1)./(2.*k.*(numZpoints.^2)));

% The Pprime expression below can be deleted:
%Pprime = exp(-1j.*(pi.^2).*(zVect(1:(numZpoints-1),1).^2)...
%    .*deltaXvect(1:(numZpoints-1),1)./(2.*k.*(numZpoints.^2)));

Pprime = verticalVector(Pprime);
%Pprime = vertcat(Pprime,0);

% Hanning window for absorption layer:
indiceValues = (verticalVector([0:1:numZpoints-Hindex]))./(numZpoints-Hindex);
%absorptionLayer = (1 +cos(pi.*indiceValues))./2;
gamma0 = 0.1;
absorptionLayer = gamma0.*(g.^(indiceValues./deltaZvect(1,1)));
indexRefraction = ones(numZpoints,1);
indexRefraction(Hindex:numZpoints,1) = indexRefraction(Hindex:numZpoints,1) ...
    + 1j.*absorptionLayer;

for x = 2:numXpoints
%     uGrid(1:(numZpoints-1),x) = inverseDiscreteSineTrans( ...
%         Pprime.*discreteSineTrans(uGrid(1:(numZpoints-1),x-1)));
%     uGrid(1:numZpoints,x) = exp(1j.*k.*(indexRefraction.^2 -1) ...
%         .*deltaXvect(1,1)./2).*uGrid(1:numZpoints,x);

    uGrid(1:(numZpoints),x)= exp(1j.*k.*(indexRefraction.^2 -1)...
        .*deltaXvect(1,1)./2) ...
        .*impedanceBoundary_SSA(uGrid(1:(numZpoints),x-1),deltaX,deltaZ,frequency,alpha);

end
uValues = uGrid;
%uValues(abs(uValues)<10^-3) = 10^-3; 
end

