% SSA_addRloss.m: Calculates the electric field (u-values) along a flat
%                 surface using the split-step algorithm. Additional loss
%                 is added, (1/r), to try to create a 3D model out of the 
%                 2D model.
% Remarks: length(xVect) >= length(zVect)             

% zFieldInit: Vector containing the initial electric field along the z-axis
% zVect: Vector containing the z-coordinates of interest
% xVect: Vector containing the x-coordinates of interest
% maxHeigthInterestZIndex: the z-index that contains the highest index of
%                          interest, above this index there will be 
%                          absorption in order to avoid reflection from the
%                          sky.
% frequency: The frequency of operation
% numPtsAbsoptionLayer: The number of height points in the absorption
%                       layer.
% antHeight: Antenna height [m]

% return: uValues: calcultaed u-values with 1/r-loss added.


function uValues = SSA_addRloss(zFieldInit,zVect, ...
    xVect,maxHeigthInterestZIndex,frequency,numPtsAbsoptionLayer,...
    antHeight)

c = 3*10^8;
lambda = c/frequency;
k = 2*pi/lambda;
numZpoints = length(zVect);
numXpoints = length(xVect);
Hindex = maxHeigthInterestZIndex;

xVect= verticalVector(xVect);
zVect = verticalVector(zVect);
deltaX = xVect(2,1)-xVect(1,1);

% Creating the a grid for the u-values 
uGrid = zeros(numZpoints,numXpoints);
uGrid(:,1)= zFieldInit;
indicesZminusOne = verticalVector(linspace(1,(numZpoints-1),(numZpoints-1)));
Pprime = exp(-1j.*(pi.^2).*(indicesZminusOne.^2)...
    .*deltaX./(2.*k.*(numZpoints.^2)));
Pprime = verticalVector(Pprime);

% Creating absorption layer:
indexRefraction = createAbsorptionLayer(zFieldInit,...
    Hindex,numPtsAbsoptionLayer);

% Calculating the field:
for x = 2:numXpoints
    uGrid(1:(numZpoints-1),x) = inverseDiscreteSineTrans( ...
        Pprime.*discreteSineTrans(uGrid(1:(numZpoints-1),x-1)));
    uGrid(1:numZpoints,x) = exp(1j.*k.*(indexRefraction.^2 -1) ...
        .*deltaX./2).*uGrid(1:numZpoints,x);
   
end

parfor x = 2:numXpoints
   distVect = sqrt(x.^2 + (zVect-antHeight).^2);
   % Adding additional loss (1/r):
    uGrid(:,x) = uGrid(:,x).*(1./distVect);
end
uValues = uGrid;

end

