% SSAirregularTerrainAbsoptionLayer.m: Uses the split-step-algoritm to
%                                      calculate the wave propagation over
%                                      irregular terrain with an
%                                      absorption layer.
% zFieldInit: Vector containing the initial electric field along the z-axis
% zVect: Vector containing the z-coordinates of interest
% xVect: Vector containing the x-coordinates of interest
% zSurfaceVect: Vector containing the z-coordinates of the surface
% maxHeigthInterestZIndex: the z-index that contains the highest index of
%                          interest, above this index there will be
%                          absorption in order to avoid reflection from the
%                          sky.
% frequency: The frequency of operation
% numPointsLayer: The number of points in the absorption layer
% zs: Transmitter antenna height

% return: uValues: The calculated u-values.

function [uValues] = SSAirregularTerrainAbsoptionLayer(zFieldInit,zVect,...
    xVect,zSurfaceVect,maxHeigthInterestZIndex,frequency,numPointsLayer,zs)

c = 3*10^8;
lambda = c/frequency;
k = 2*pi/lambda;
zVectInit = zVect;
numZpoints = length(zVect);
numZpointsInit = numZpoints;
numXpoints = length(xVect);
zVect = verticalVector(zVect);
xVect = verticalVector(xVect);
zSurfaceVect = verticalVector(zSurfaceVect);
Hindex = maxHeigthInterestZIndex;


deltaZ = zVect(2,1) - zVect(1,1);
deltaX = xVect(2,1) - xVect(1,1);

% Finding the maximum height of the surface:
[maxZ, indexMaxZ] = max(zSurfaceVect);

% The number of points to add:
addVect =verticalVector([zVect(numZpoints,1)+deltaZ:deltaZ:...
    (zVect(numZpoints,1)+ maxZ+deltaZ)]);
zVect = vertcat(zVect,addVect);
numZpoints = length(zVect);
% Mapping the surface-vector to correspond with the heights in zVect
% The values of the new surface vector contains the indices corresponding
% to the height in the zVect
zSurfaceIndices = zeros(numXpoints,1);
for x = 1:numXpoints
    zSurfaceIndices(x,1) = find(zVect >=zSurfaceVect(x,1),1,'first');
end

% Creating absorption layer:
indexRefraction = createAbsorptionLayer(zFieldInit,...
    Hindex,numPointsLayer);

uGrid = zeros(numZpoints,numXpoints);
uGrid(1:numZpointsInit,1)= zFieldInit;
indicesZminusOne = verticalVector(linspace(1,(numZpointsInit-1), ...
    (numZpointsInit-1)));
Pprime = exp(-1j.*(pi.^2).*(indicesZminusOne.^2)...
    .*deltaX./(2.*k.*(numZpointsInit.^2)));
Pprime = verticalVector(Pprime);

% % Creating the vGrid:
% vGrid = zeros(numZpointsInit,numXpoints);
% 
% % Creating the vector containing the slopes of the terrain
% alphaVect =  (zSurfaceVect(2:numXpoints,1)- ...
%     zSurfaceVect(1:(numXpoints-1),1))./deltaX;
% alphaVect = vertcat(0,alphaVect);
% 
% zeta = zeros(numZpoints,1);
% 
% vGrid(:,1) = uGrid(zSurfaceIndices(1):...
%     (zSurfaceIndices(1)+numZpointsInit-1),1);


% The staircase model:
zVect = zVectInit;
numZpoints = length(zVect);
uGrid = zeros(numZpoints,numXpoints);
uGrid(:,1)= zFieldInit;
indexRefraction = createAbsorptionLayer(zFieldInit,...
    Hindex,numPointsLayer);
indicesZminusOne = verticalVector(...
    linspace(1,(numZpoints-1),(numZpoints-1)));
Pprime = exp(-1j.*(pi.^2).*(indicesZminusOne.^2)...
    .*deltaX./(2.*k.*(numZpoints.^2)));
Pprime = verticalVector(Pprime);

% Calculating the field:
for x = 2:numXpoints
     numZpoints = length(zVect);
    uGrid(1:(numZpoints-1),x) = inverseDiscreteSineTrans( ...
        Pprime.*discreteSineTrans(uGrid(1:(numZpoints-1),x-1)));
     uGrid(1:numZpoints,x) = exp(1j.*k.*(indexRefraction.^2 -1) ...
        .*deltaX./2).*uGrid(1:numZpoints,x);
    
    % Adjusting to the surface:
    if zSurfaceIndices(x,1) > 1
        uGrid(1:(zSurfaceIndices(x,1)-1),x) = 0;       
    end

end
uValues = uGrid;
end
