% freeSpaceLoss.m: Calculates the free-space loss. Adjusts the beam pattern 
%                  according to the parameters (works only for rx = tx).
% tx: Transimtter antenna height
% rx: Receiver antenna height
% distance: The distance between the antennas, ignoring height difference
% eField: Calculated electric field along the surface at a given height,
%         zs, may contain multiple heights of consideration
% zs: The height eField is taken from
% beta: Beam-width [rad]
% frequency: The frequency of operation
% simulationType: Cell-array containing strings with the name of the
%                 simulation types used.
% plotTitle: The title of the plot; Vector of strings.
% saveFig: String of path and filename for saving the plot result. To not
%          save the result, saveFig should be an empty string (''). File
%          extension: .png


function freeSpaceLoss_beamParam(A,tx,rx,xVect,eField,zs,...
    beta,frequency,simulationType, plotTitle,saveFig)
c = 3*10^8;
lambda = c/frequency;
k = 2*pi/lambda;
numXpoints = length(xVect);
xVect = verticalVector(xVect);
numXpoints = length(xVect);
tx_x = xVect(1,1);
rx_x = xVect(numXpoints,1);
tx_z = tx;
rx_z = rx;
% Finding the dimensions of the incoming e-field array:
[lines,columns] = size(eField);
if(lines>columns)
    eField = eField';
    numCases = columns;
else
    numCases = lines;
end
cmap = hsv(numCases+1);

plotNames = cell(1+length(simulationType),1);
plotNames{1,1} = 'Free-space Loss - analytical model';
for i = 1: length(simulationType);
    plotNames{i+1,1} = simulationType{i};
end
% Calculating the received fields:
a = tx_z/rx_z;

xVectAux = xVect(2:numXpoints,1);
thetaDir = atan((rx_z-tx_z)./(abs(tx_x-xVectAux)));
Adirect = A.*exp(-2.*log10(2).*((thetaDir/beta).^2));

Atotal = Adirect; 
Pl = (lambda./(4.*pi.*(abs(tx_x-xVectAux)))).*((Atotal./Adirect).^2);

% Save the figure?
saveFigVal = isempty(saveFig);

switch saveFigVal
    case 1
        % Not save figure
        figure()
    case 0
        % Save figure
        fig = figure('visible','on');
end


% Calculating the plane earth loss (path loss for plane earth):

Pl_dB = 20.*log10(verticalVector(Pl(9:numXpoints-1,1)));
semilogx(xVect(9:numXpoints-1,1),Pl_dB,'Color',cmap(1,:));
xlabel('Distance [m]');
ylabel('Path Loss [-dB]');

for i = 1: numCases
    hold on
    
    % The simulated field:
    pl1 = verticalVector(20.*log10(abs(eField(i,:))));
    pl2 = - 10.*log10(xVect);
    
    pathLoss = pl1 + pl2;
    semilogx(xVect(10:numXpoints,1),pathLoss(10:numXpoints,1),...
        'Color',cmap(i+1,:)); 
    
end
legend(plotNames);
title(plotTitle);
grid on

switch saveFigVal
    case 0
        % Save figure
        titleFig = verticalVector(saveFig)';
        regexprep(titleFig,' ','_');
        regexprep(titleFig,'\','_');
        regexprep(titleFig,':','_');
        regexprep(titleFig,'=','_');
             
        if (isempty(strfind(titleFig,'.png')) == 1)
            % File extension needs to be added.
            titleFig = horzcat(titleFig,'.png');
        end
            
        saveas(fig,titleFig,'png');
end

end



