% getUTDfield.m: Calculates the e-field for a given set of 
%                parameters using the UTD method from the ITU-R report.
% Assumption: - Assuming there is one wedge only.
% Xn: Vector containing x-values, for description of the terrain. Does not
%     need to fulfill Nyquist's sampling criteria.
% Zn: Vector contaning z-values, for description of the terrain. Does not
%     need to fulfill Nyquist's sampling criteria.
% deltaX: The desired value of deltaX. Leave deltaX=0 for default value of
%         deltaX. Default value: the smallest value that fulfills Nyquist's
%         sampling criteria.
% tx_x: x-coordinate of the transmitter.
% tx_z: z-coordinate of the transmitter.
% rx_xArr: x-coordinates of the receiver points.
% rx_zArr: z-coordinates of the receiver points.
% frequency: the frequency of operation.
% e0: Transmitter antenna gain.

% Return:
%        xVect: Surface coordinates along the x-axis.
%        zVect: Surface coordinates along the z-axis.
%        eUTD: Array containing the calculated e-field at the specified
%              points.


function [xVect,zVect,eUTD] = getUTDfield(Xn,Zn,deltaX,tx_x,tx_z,rx_xArr,rx_zArr,frequency,e0)
c=3.*10^8;
lambda = c./frequency;
k = 2.*pi/lambda;

% Creating vectors of the terrain profile, fulfills Nyquist's sampling
% criteria:
[xVect,zVect] = interpolate(Xn,Zn, frequency,'linear', deltaX);
xVect = verticalVector(xVect);
%numXpoints = length(xVect);

% tx_x = xVect(1,1);
% rx_x = xVect(numXpoints,1);
% tx_z = tx;
% rx_z = rx;

% Calculating the diffraction coefficient:

% The an, bn, cn, and dn coefficients, ITU-R report p.4
an_coeffs = [+1.595769140, -0.000001702, -6.808568854, -0.000576361, ...
    +6.920691902, -0.016898657, -3.050485660, -0.075752419,+0.850663781 ...
    -0.025639041, -0.150230960, +0.034404779];

bn_coeffs = [-0.000000033, +4.255387524, -0.000092810, -7.780020400, ...
    -0.009520895, +5.075161298, -0.138341947, -1.363729124,-0.403349276 ...
    +0.702222016, -0.216195929, +0.019547031]; 

cn_coeffs = [+0.000000000, -0.024933975, +0.000003936, +0.005770956 ...
    +0.000689892, -0.009497136, +0.011948809, -0.006748873,+0.000246420 ...
    +0.002102967, -0.001217930, +0.000233939];

dn_coeffs = [+0.199471140, +0.000000023, -0.009351341, +0.000023006, ...
    +0.004851466, +0.001903218, -0.017122914, +0.029064067,-0.027928955 ...
    +0.016497308, -0.005598515, +0.000838386];

% Finding the parameters:

%s1:
index = find(zVect == max(zVect(:)));

s1 = sqrt( ((xVect(index)-tx_x).^2) + ((zVect(index)-tx_z).^2));

%s2:
s2 = verticalVector(sqrt( ((rx_xArr-xVect(index)).^2) + ((zVect(index)-rx_zArr).^2)));

% The diffraction coefficient: 

% Incidence angle (phi1), measured from fase 0:
theta1aux = atan(zVect(index)/(xVect(index)-tx_x));
theta1 = (pi/2) - theta1aux;
phi1 = asin(sin(theta1)*tx_z/s1);

% Diffraction angle (phi2), measured from fase 0:
alpha1aux = atan((xVect(index)-tx_x)/zVect(index));
alpha2aux = atan((rx_xArr-xVect(index))/zVect(index));
theta2aux = atan(zVect(index)/(rx_xArr-xVect(index)));
theta2 = verticalVector((pi./2) - theta2aux);
alpha3aux = asin(sin(theta2).*zVect(index)./s2);
phi2 = (2*pi) - alpha1aux - alpha2aux - alpha3aux;

% External wedge angle:
phiWedge = (2.*pi) - alpha1aux - alpha2aux;
n = phiWedge./pi;

% The other coefficients:
L = (s1.*s2./(s1+s2));

betaPlus = phi2 + phi1;
betaMinus = phi2 - phi1;

Nplus = round((betaPlus + pi)./(2.*n.*pi));
Nminus = round((betaMinus - pi)./(2.*n.*pi));

aPlus = 2.*cos(((2.*n.*pi.*Nplus)-betaPlus)./2);
aMinus =  2*cos(((2.*n.*pi.*Nminus)-betaMinus)./2);

% Horizontal polarization with respect to the ground. The ground is almost
% horizonal due to small hegiht differences.
R = -1; % reflection coefficient. For both sides of the wedge. ? TODO!!

% Creating array containing
Darr = zeros(length(s2),1);

for m = 1: length(s2)
% The calculations:
arg1 = k.*L(m,1).*aPlus(m,1).*betaMinus(m,1);
switch arg1
    case arg1 < 4
        coeff_sum = sum( (an_coeffs + 1j.*bn_coeffs).*((arg1./4).^n(m,1)));
        A_arg1 = ((1-1j)./2) - exp(-1j.*arg1).*sqrt(arg1./4)*coeff_sum;
    otherwise
        coeff_sum = sum( (cn_coeffs + 1j.*dn_coeffs).*((4./arg1).^n(m,1)));
        A_arg1 = -exp(-1j.*arg1).*sqrt(4./arg1).*coeff_sum;
end


F_arg1 = 1j.*2.*sqrt(arg1).*exp(1j.*arg1).*((sqrt(pi./8)-(1-1j)) - ...
    (sqrt(pi./2).*A_arg1));
coeff1 = cot((pi+betaMinus(m,1))./(2.*n(m,1))).*F_arg1;

arg2 = k.*L(m,1).*aMinus(m,1).*betaMinus(m,1);
switch arg2
    case arg2 < 4
        coeff_sum = sum( (an_coeffs + 1j.*bn_coeffs).*((arg2./4).^n(m,1)));
        A_arg2 = ((1-1j)./2) - exp(-1j.*arg2).*sqrt(arg2./4).*coeff_sum;
    otherwise
        coeff_sum = sum( (cn_coeffs + 1j.*dn_coeffs).*((4./arg1).^n(m,1)));
        A_arg2 = -exp(-1j.*arg2).*sqrt(4./arg2).*coeff_sum;
end

F_arg2 = 1j.*2.*sqrt(arg2).*exp(1j.*arg2).*((sqrt(pi./8)-(1-1j)) - ...
    (sqrt(pi/2)*A_arg2));
coeff2 = cot((pi-betaMinus(m,1))./(2.*n(m,1))).*F_arg2;

arg3 = k.*L(m,1).*aMinus(m,1).*betaPlus(m,1);
switch arg3
    case arg3 < 4
        coeff_sum = sum( (an_coeffs + 1j.*bn_coeffs).*((arg3./4).^n(m,1)));
        A_arg3 = ((1-1j)./2) - exp(-1j.*arg3).*sqrt(arg3./4).*coeff_sum;
    otherwise
        coeff_sum = sum( (cn_coeffs + 1j.*dn_coeffs).*((4./arg3).^n(m,1)));
        A_arg3 = -exp(-1j.*arg3).*sqrt(4./arg3).*coeff_sum;
end

F_arg3 = 1j.*2.*sqrt(arg3).*exp(1j.*arg3).*((sqrt(pi./8)-(1-1j)) - ...
    (sqrt(pi./2).*A_arg3));
coeff3 = cot((pi-betaPlus(m,1))./(2.*n(m,1))).*F_arg3;

arg4 = k.*L(m,1).*aPlus(m,1).*betaPlus(m,1);
switch arg4
    case arg4 < 4
        coeff_sum = sum( (an_coeffs + 1j.*bn_coeffs).*((arg4./4).^n(m,1)));
        A_arg4 = ((1-1j)./2) - exp(-1j.*arg4).*sqrt(arg4./4).*coeff_sum;
    otherwise
        coeff_sum = sum( (cn_coeffs + 1j.*dn_coeffs).*((4./arg4).^n(m,1)));
        A_arg4 = -exp(-1j.*arg4).*sqrt(4./arg4).*coeff_sum;
end

F_arg4 = 1j.*2.*sqrt(arg4).*exp(1j.*arg3)*((sqrt(pi./8)-(1-1j)) - ...
    (sqrt(pi./2).*A_arg4));
coeff4 = cot((pi+betaPlus(m,1))/(2.*n(m,1))).*F_arg4;


Daux = coeff1 + coeff2 + R.*coeff3 + R.*coeff4;
Darr(m,1) = Daux;
end
D = Daux.*(-exp(-1j.*pi./4)./(2.*n(m,1).*sqrt(2.*pi.*k))); % The diffraction coefficient

eUTD = e0.*(exp(-1j*k*s1)./s1)*D*sqrt(s1./(s2.*(s1+s2))).*exp(-1j*k*s2);

end