% getVerticalValues.m: Calculates the values in the "vertical" direction,
%                      the direction perpendicular to the downward inclined
%                      plane.

% xDiff: The distance difference in the x-direction
% zDiff: The height difference in the z-direction
% deltaX: The step size along the x-axis
% deltaZ: The step size along the z-axix
% xVect: The vector containing the x-values
% xDistCompare: The distance along the x-axis (if the surface was flat) where
%             the field comparison "takes place".
% maxZ: The maximum height for field comparison.
% uValues: Calculated grid of u-values.
% directionIncl: The direction of inclination: 'up' or 'down'

% return: zValues: The field values in the ''vertical'' direction



function zValues = getVerticalValues(Xn,Zn,xDiff,zDiff,deltaX,deltaZ,...
    xVect,xDistCompare,maxZ,uValues, directionIncl)

zValues = verticalVector([0:deltaZ:maxZ]); % counts from 0 to maxZ
xStartIndex =  find(xVect >= xDistCompare ,1);

switch directionIncl
    
    case 'down'
        thetaAux = atan(zDiff/xDiff);
        xDist2 = xDistCompare*cos(thetaAux);
        xStartIndex =  find(xVect >= xDist2 ,1);
        
        theta = atan(xDiff/abs(zDiff));
        phi = (2*pi) - pi- theta;
        % The height difference at the xDistCompare along the plane to the flat
        % surface:
        %zDiffAdd = round(abs(zDiff) - abs((xDistCompare*sin(thetaAux))));
        zDiffAdd = round(abs(Zn(1)) - abs((xDistCompare*sin(thetaAux))));
        
    case 'up'
        theta = atan(zDiff/xDiff);
        phi = pi - (pi/2)- theta;
        xDist2 = xDistCompare*cos(theta);
        xStartIndex =  find(xVect >= xDist2 ,1);
        
        % The height at which is the plane is situated at, at xDistCompare:
        % (the coordinate)
        zDiffAdd = round(xDistCompare*sin(theta)/deltaZ);
        zDiffAdd_noRounded = xDistCompare*sin(theta)/deltaZ;
        %Finding the endpoint of the vector
        vectorLength = maxZ;
        totLength = vectorLength + zDiffAdd_noRounded;
        xDiffLength = totLength.*cos(phi);
        xDiffCoord = round(xStartIndex - (xDiffLength./deltaX));
        zHeight =  totLength.*sin(phi);
        normVect = [(zHeight-zDiffAdd_noRounded) -xDiffLength];
        % NB: Convention for this thesis: [z x]
        
        
end

xCoordVect = zeros(length(zValues),1);
zCoordVect = zeros(length(zValues),1);
for i = 1:length(zValues)
    xCoord = xStartIndex - round(zValues(i).*deltaX.*cos(phi));
    zCoord = zDiffAdd + round(zValues(i).*deltaZ.*sin(phi));
    if(zCoord == 0)
        zCoord = 1;
    end
    
    zValues(i,1) = uValues(zCoord,xCoord);
    
    
    xCoordVect(i,1) = xCoord;
    zCoordVect(i,1) = zCoord;
end

% % Verifying that the length of normVect is vectorLength:
% norm(normVect)
% % Verifying the dot product between the normal and the plane (should be 90
% % deg):
% surfaceVect = [(Zn(length(Zn))-Zn(1)) -(Xn(length(Xn))-Xn(1)) ];
% dotRes = dot(normVect,surfaceVect)/(norm(normVect)*norm(surfaceVect))
%
% figure()
% plot(Xn,Zn);
% hold on
% plot([xStartIndex xDiffCoord],[zDiffAdd zHeight]);

% % Finding the length of the segment between the plane and flat surface, if
% % the normal vector was extended.
% xPosNorm = xDistCompare/cos(theta);



%figure()
plot(Xn,Zn,'k')
hold on
plot(xCoordVect,zCoordVect,'k')
grid on

% Calculating the dot product:
surfaceVect = [-(Xn(length(Xn))-Xn(1)) (Zn(length(Zn))-Zn(1))];
ninetyDegVect = [-(xCoordVect(length(xCoordVect))-xCoordVect(1)) ...
    (zCoordVect(length(zCoordVect))-zCoordVect(1))];

% The result of the dot product should be zero (or very close to)
dotResult = dot(surfaceVect,ninetyDegVect)/(norm(surfaceVect).*norm(ninetyDegVect))




end
