% pathLossFlat_Indra.m: Vertical comparison between the relative field
%                       strengths of the simulated fields and the results 
%                       from a given file.

% rx_xArr: Receiver antenna (obseravation points) coordinates along the
%          x-axis.
% rx_zArr: Receiver antenna (observation points) coordinates along the
%          z-axis.
% eField: Calculated electric field at the coordinates to rx_xArr and
%         rx_zArr.
% frequency: The frequency of operation
% simulationType: Cell-array containing strings with the name of the
%                 simulation types used.
% plotTitle: Title of the plot.
% saveFig: file name of the plot, leave blank (' ') if the plot should not
%          be saved.
% compareFile: File name and relative path if necessary of the file to 
%              compare results with.


function pathLossFlat_Indra(rx_xArr,rx_zArr,eField,...
    frequency,simulationType, plotTitle,saveFig,compareFile,compareHeight)
c = 3*10^8;
lambda = c/frequency;
k = 2*pi/lambda;
columnNumber = 3;
[eFieldValues,zVectIndra]=importFieldResultsFromFile(compareFile,...
    columnNumber);
eFieldValues = verticalVector(eFieldValues);
zVectIndra = verticalVector(zVectIndra);

indexMaxIndra = find(zVectIndra >= compareHeight,1);
zVectIndra = zVectIndra(1:indexMaxIndra,1);
eFieldValues = eFieldValues(1:indexMaxIndra,1);

rx_xArr = verticalVector(rx_xArr);
rx_zArr = verticalVector(rx_zArr);
 
indexMaxZ = find(rx_zArr >= compareHeight,1); %240 ,1);
rx_xArr = rx_xArr(1:indexMaxZ,1);
rx_zArr = rx_zArr(1:indexMaxZ,1);
eField = eField(:,1:indexMaxZ,1);

rx_zArr = verticalVector(rx_zArr);
numRxPoints = length(rx_zArr);
thisFontSize = 10;



% Finding the dimensions of the incoming e-field array:
[lines,columns] = size(eField);
if(lines>columns)
    eField = eField';
    numCases = columns;
else
    numCases = lines;
end
cmap = hsv(numCases+1);

plotNames = cell(1+length(simulationType),1);
plotNames{1,1} = 'E-field Indra';
for i = 1: length(simulationType);
    plotNames{i+1,1} = simulationType{i};
end

% Save the figure?
saveFigVal = isempty(saveFig);

switch saveFigVal
    case 1
        % Not save figure
        figure()
    case 0
        % Save figure
        fig = figure('visible','on');
end

% Plotting the E-field calculated at indra:
lineWidth  = 1;
Pl =eFieldValues; % In dB already 
Pl_dB =Pl; 
hl1 = line(zVectIndra(7:length(zVectIndra),1),...
    Pl_dB(7:length(zVectIndra),1),'Color',cmap(1,:),'LineWidth',lineWidth);
ax1 = gca;
set(ax1,'XColor','k','YColor','k')
%legend(plotNames{1,1})
set(ax1, 'Box', 'off')
set(ax1, 'Color', 'none')
maxImportedEfield = max(eFieldValues);
set(gca,'FontSize',thisFontSize) 
xlabel('Height [m]');
ylabel('Relative field strength [dB]');

ax2 = axes('Position',get(ax1,'Position'),...
           'XAxisLocation','top',...
           'YAxisLocation','right',...
           'Color','none',...
           'XColor','k','YColor','k');

for i = 1: numCases
    hold on
    
    % The simulated fields:
    pl1 = smooth(verticalVector(20.*log10(abs(eField(i,:)))));
    maxThisField = max(pl1);
    maxDiff = maxImportedEfield - maxThisField;
    pl2 = 0; 
    pathLoss = pl1 + pl2; 

    pathLoss = pathLoss + maxDiff;
        
    hl2 = line(rx_zArr(4:length(rx_zArr),1),pathLoss(4:length(rx_zArr)),...
        'Color',cmap(5-i+1,:),'Parent',ax2,'LineWidth',lineWidth);
end

set(ax2, 'Box', 'off')
set(gca,'FontSize',thisFontSize) 
legend(ax1,plotNames{1,1},'Location','South')
legend(ax2,plotNames{2:numCases+1,1},'Location','SouthEast')
title(plotTitle);

grid on
linkaxes([ax2 ax1],'y');

switch saveFigVal
    case 0
        % Save figure
        titleFig = verticalVector(saveFig)';
        regexprep(titleFig,' ','_');
        regexprep(titleFig,'\','_');
        regexprep(titleFig,':','_');
        regexprep(titleFig,'=','_');
        
        if (isempty(strfind(titleFig,'.png')) == 1)
            % File extension needs to be added.
            titleFig = horzcat(titleFig,'.png');
        end
        
        saveas(fig,titleFig,'png');
        titleFig = titleFig(1:(length(titleFig)-4));
        titleFig = horzcat(titleFig,'.pdf');
        %print (fig, '-dpdf', titleFig);
        save2pdf(titleFig);
end

end
