% pathLossFlat.m: Calculates the path loss along a flat surface, used for
%                 comparing with plane earth loss model, assuming zero
%                 reflection loss. Adjusts the beam pattern according to
%                 the parameters (works only for rx = tx).
% tx: Transimtter antenna height
% rx: Receiver antenna height
% eField: Calculated electric field along the surface at a given height,
%         zs, may contain multiple heights of consideration
% zs: The height eField is taken from
% beta: Beam-width [rad]
% frequency: The frequency of operation
% simulationType: Cell-array containing strings with the name of the
%                 simulation types used.
% plotTitle: The title of the plot; Vector of strings.
% saveFig: String of path and filename for saving the plot result. To not
%          save the result, saveFig should be an empty string (''). File
%          extension: .png


function pathLossFlat_beamParam(A,tx,rx,xVect,eField,zs,...
    beta,frequency,simulationType, plotTitle,saveFig)
c = 3*10^8;
lambda = c/frequency;
k = 2*pi/lambda;
numXpoints = length(xVect);
xVect = verticalVector(xVect);
numXpoints = length(xVect);
tx_x = xVect(1,1);
rx_x = xVect(numXpoints,1);
tx_z = tx;
rx_z = rx;
% Finding the dimensions of the incoming e-field array:
[lines,columns] = size(eField);
if(lines>columns)
    eField = eField';
    numCases = columns;
else
    numCases = lines;
end
cmap = hsv(numCases+1);

plotNames = cell(1+length(simulationType),1);
plotNames{1,1} = 'Plane Earth Loss';
for i = 1: length(simulationType);
    plotNames{i+1,1} = simulationType{i};
end
% Calculating the received fields:
a = tx_z/rx_z;

xVectAux = xVect(2:numXpoints,1);
thetaDir = atan((rx_z-tx_z)./(abs(tx_x-xVectAux)));
Adirect = A.*exp(-2.*log10(2).*((thetaDir/beta).^2));

% The reflected field:

% Find theta:
d_tx = a.*abs(tx_x-xVectAux)./(1+a);
thetaRefl = (pi/2) - atan(d_tx./tx_z);
Areflect = A.*exp(-2.*log10(2).*((thetaRefl./beta).^2));

Atotal = Adirect + Areflect.*exp(1j.*k.*rx_z.*tx_z./(abs(tx_x-xVectAux)));

Pl = (lambda./(4.*pi.*(abs(tx_x-xVectAux)))).*(abs(Atotal./Adirect));

% Save the figure?
saveFigVal = isempty(saveFig);

switch saveFigVal
    case 1
        % Not save figure
        figure()
    case 0
        % Save figure
        fig = figure('visible','on');
end


% Calculating the plane earth loss (path loss for plane earth):
Pl_dB = 20.*log10(verticalVector(Pl(9:numXpoints-1,1)));
semilogx(xVect(9:numXpoints-1,1),Pl_dB,'Color',cmap(1,:));
xlabel('Distance [m]');
ylabel('Path Loss [-dB]');
for i = 1: 2:(numCases-1)
    % Calculating plane earth loss from the simulated results:
    hold on
    
    pl1 = verticalVector(20.*log10(abs(eField(i,:))));
    pl2 = - 10.*log10(xVect);
    
    pathLoss = pl1 + pl2; 
    semilogx(xVect(10:numXpoints,1),pathLoss(10:numXpoints,1),...
        'Color',cmap(i+1,:)); 
    
    pl1 = verticalVector(20.*log10(abs(eField(i+1,:))));
    pl2 = - 10.*log10(xVect);
    
    pathLoss = pl1 + pl2; 
    semilogx(xVect(10:numXpoints,1),pathLoss(10:numXpoints,1),'--',...
        'Color',cmap(i+1,:)); 
   
end

legend(plotNames,'Location','SouthWest');
title(plotTitle);
grid on

switch saveFigVal
    case 0
        % Save figure
        titleFig = verticalVector(saveFig)';
        regexprep(titleFig,' ','_');
        regexprep(titleFig,'\','_');
        regexprep(titleFig,':','_');
        regexprep(titleFig,'=','_');
             
        if (isempty(strfind(titleFig,'.png')) == 1)
            % File extension needs to be added.
            titleFig = horzcat(titleFig,'.png');
        end
            
        saveas(fig,titleFig,'png');
        %print (fig, '-dpdf', titleFig);
        titleFig = titleFig(1:(length(titleFig)-4));
        titleFig = horzcat(titleFig,'.pdf');
        save2pdf(titleFig);
end

end



