% pathLossFlat_zDirection.m: Calculates the path loss at a flat surface, used for
%                 comparing with plane earth loss model, assuming zero
%                 reflection loss. Adjusts the beam pattern according to
%                 the parameters (works only for rx = tx). Compares in the
%                 height direction at a given distance along the surface.
% tx: Transimtter antenna height
% eField: Calculated electric field along the surface at a given height,
%         zs, may contain multiple heights of consideration
% deltaZ: The step size in the vertical direction
% xDist: Distance along the surface at which the fields are to be compared.
% zDist: The highest height of consideration for comparison of fields
% beta: Beam-width [rad]
% frequency: The frequency of operation
% simulationType: Cell-array containing strings with the name of the
%                 simulation types used.
% plotTitle: The title of the plot; Vector of strings.
% saveFig: String of path and filename for saving the plot result. To not
%          save the result, saveFig should be an empty string (''). File
%          extension: .png


function pathLossFlat_zDirection(A,tx,eField,deltaZ,xDist,zDist,...
    beta,frequency,simulationType, plotTitle,saveFig)
c = 3*10^8;
lambda = c/frequency;
k = 2*pi/lambda;
tx_x = 0;
rx_x =xDist;
tx_z = tx;
% Finding the dimensions of the incoming e-field array:
[lines,columns] = size(eField);
if(lines>columns)
    eField = eField';
    numCases = columns;
else
    numCases = lines;
end
cmap = hsv(numCases+1);

plotNames = cell(1+length(simulationType),1);
plotNames{1,1} = 'Plane Earth Loss';
for i = 1: length(simulationType);
    plotNames{i+1,1} = simulationType{i};
end
% Calculating the received fields:


zVextAux = verticalVector([1:deltaZ:zDist]);
thetaDir = atan((zVextAux-tx_z)./(abs(tx_x-xDist)));
Adirect = A.*exp(-2.*log10(2).*((thetaDir/beta).^2));
a = tx_z./zVextAux;

% The reflected field:

% Find theta:
d_tx = a.*abs(tx_x-xDist)./(1+a);
thetaRefl = (pi/2) - atan(d_tx./tx_z);
Areflect = A.*exp(-2.*log10(2).*((thetaRefl./beta).^2));



Atotal = Adirect + Areflect.*exp(1j.*k.*zVextAux.*tx_z./(abs(tx_x-xDist)));
%Pl = Atotal;
%Pl = (lambda./(4.*pi.*(abs(tx_x-xVectAux)))).*((Atotal./Adirect).^2);
Pl = (lambda./(4.*pi.*(abs(tx_x-xDist)))).*(abs(Atotal./Adirect));

%simulationType = verticalVector(simulationType);
%plotNames = vertcat('Plane Earth Loss', simulationType);

% Save the figure?
saveFigVal = isempty(saveFig);

switch saveFigVal
    case 1
        % Not save figure
        figure()
    case 0
        % Save figure
        fig = figure('visible','on');
end


% Calculating the plane earth loss (path loss for plane earth):

Pl_dB = 20.*log10(verticalVector(Pl(1:length(zVextAux),1)));
semilogx(zVextAux,Pl_dB,'Color',cmap(1,:));
xlabel('Height [m]');
ylabel('Path Loss [-dB]');
%legend('Plane Earth Loss');

for i = 1: numCases
    hold on
    
    % Newly inserted:
    pl1 = verticalVector(20.*log10(abs(eField(i,:))));
    %pl2 = - 10.*log10(xVect);
    %pl3 = pl1 + pl2;
    pathLoss = pl1;% + pl2; %20.*log10(abs(eField(i,:))) - 10.*log10(xVect); % - 20.*log10(4.*pi) + 30.*log(lambda);
    semilogx(zVextAux,pathLoss(2:length(zVextAux),1),'Color',cmap(i+1,:)); %,'LineWidth',1*(numCases+1-i));
    % Calculating the path loss for the incoming eField
    %semilogx(xVect,20.*log10(abs(eField(i,:))),colors(i));
    %legend(simulationType(i));
end
legend(plotNames);
title(plotTitle);
grid on

switch saveFigVal
    case 0
        % Save figure
        titleFig = verticalVector(saveFig)';
        regexprep(titleFig,' ','_');
        regexprep(titleFig,'\','_');
        regexprep(titleFig,':','_');
        regexprep(titleFig,'=','_');
             
        if (isempty(strfind(titleFig,'.png')) == 1)
            % File extension needs to be added.
            titleFig = horzcat(titleFig,'.png');
        end
            
        saveas(fig,titleFig,'png');
        print (fig, '-dpdf', titleFig);
end

end



