% pathLossWedge_Indra.m: Vertical comparison between the relative field
%                       strengths of the simulated fields and the results 
%                       from a given file. Shiftes the fields to be 
%                       alligned with the maximim value of the results
%                       imported from the file.
% Xn: Vector of x-values
% Zn: Vector of corresponding z-values
% deltaX: spacing between the sampling points along the x-axis, if set to 0
%         the spacing is according to Nyquist's theorem.
% tx: Transimtter antenna height
% rx_xArr: Receiver antenna (obseravation points) coordinates along the
%          x-axis.
% rx_zArr: Receiver antenna (observation points) coordinates along the
%          z-axis.
% distance: The distance between the antennas, ignoring height difference
% eField: Calculated electric field at the coordinates to rx_xArr and
%         rx_zArr.
% zs: The height eField is taken from
% frequency: The frequency of operation
% simulationType: Cell-array containing strings with the name of the
%                 simulation types used.
% plotTitle: Title of the plot.
% saveFig: file name of the plot, leave blank (' ') if the plot should not
%          be saved.
% compareFile: File name and relative path if necessary of the file to
%              compare results with. Leave empty (' ') if there are no
%              results to compare with.


function pathLossWedge_Indra(Xn,Zn,deltaX,A,tx,rx_xArr,rx_zArr,eField,...
    zs,beta,frequency,simulationType, plotTitle,saveFig,compareFile)
c = 3*10^8;
lambda = c/frequency;
k = 2*pi/lambda;
columnNumber = 3;
thisFontSize = 10;

tx_x = Xn(1,1); 
tx_z = tx;

indexMaxZ = find(rx_zArr >= 250 ,1);
rx_xArr = rx_xArr(1:indexMaxZ);
rx_zArr = rx_zArr(1:indexMaxZ);
eField = eField(:,1:indexMaxZ);


rx_zArr = verticalVector(rx_zArr);
numRxPoints = length(rx_zArr);


% Finding the dimensions of the incoming e-field array:
[lines,columns] = size(eField);
if(lines>columns)
    eField = eField';
    numCases = columns;
else
    numCases = lines;
end
cmap = hsv(numCases+1);

% Save the figure?
saveFigVal = isempty(saveFig);

switch saveFigVal
    case 1
        % Not save figure
        figure()
    case 0
        % Save figure
        fig = figure('visible','on');
end

switch compareFile
    
    case ' '
        % Do not try to import anyting
        plotNames = cell(length(simulationType),1);
        for i = 1: length(simulationType);
            plotNames{i,1} = simulationType{i};
        end
        
        xlabel('Height [m]');
        ylabel('Relative field strength [dB]');
        for i = 1: numCases
            hold on
            pl1 = smooth(verticalVector(20.*log10(abs(eField(i,:)))));
            pl2 = 0;
            pathLoss = pl1 + pl2;
            hl2 = line(rx_zArr(:,1),pathLoss,'Color',cmap(i+1,:));
        end
        legend(plotNames,'Location','SouthEast')
        title(plotTitle);
        grid on
        
        
    otherwise
        [eFieldValues,zVectIndra]=importFieldResultsFromFile(...
            compareFile,columnNumber);
        
        plotNames = cell(1+length(simulationType),1);
        plotNames{1,1} = 'E-field Indra';
        for i = 1: length(simulationType);
            plotNames{i+1,1} = simulationType{i};
        end
        
        % Plotting the E-field calculated at indra:
        lineWidth  = 1;
        Pl =eFieldValues; % In dB already
        Pl_dB =Pl(4:length(Pl));
        maxValueCompareField = max(Pl_dB);
        hl1 = line(zVectIndra(4:length(Pl)),Pl_dB,'Color',cmap(1,:), ...
            'LineWidth',lineWidth);
        ax1 = gca;
        set(ax1,'XColor','k','YColor','k')
        set(ax1, 'Box', 'off')
        set(ax1, 'Color', 'none')
        set(gca,'FontSize',thisFontSize)
        xlabel('Height [m]');
        ylabel('Relative field strength [dB]');
        ax2 = axes('Position',get(ax1,'Position'),...
            'XAxisLocation','top',...
            'YAxisLocation','right',...
            'Color','none',...
            'XColor','k','YColor','k');
        
        for i = 1: numCases
            hold on
            pl1 = smooth(verticalVector(20.*log10(abs(eField(i,:)))));
            maxValueEfield = max(pl1);
            maxValDiff = maxValueCompareField - maxValueEfield;
            pl2 = 0;
            pathLoss = pl1 + pl2;
            pathLoss = pathLoss + maxValDiff;
            hl2 = line(rx_zArr(:,1),pathLoss,'Color',cmap(i+1,:),...
                'Parent',ax2,'LineWidth',lineWidth);
        end
        set(ax2, 'Box', 'off')
        set(gca,'FontSize',thisFontSize) 
        legend(ax1,plotNames{1,1},'Location','East'); %'South')
        legend(ax2,plotNames{2:numCases+1,1},'Location','SouthEast')
        title(plotTitle);
        grid on
        linkaxes([ax2 ax1],'y');
        
end

switch saveFigVal
    case 0
        % Save figure
        titleFig = verticalVector(saveFig)';
        regexprep(titleFig,' ','_');
        regexprep(titleFig,'\','_');
        regexprep(titleFig,':','_');
        regexprep(titleFig,'=','_');
        
        if (isempty(strfind(titleFig,'.png')) == 1)
            % File extension needs to be added.
            titleFig = horzcat(titleFig,'.png');
        end
        
        saveas(fig,titleFig,'png');
        titleFig = titleFig(1:(length(titleFig)-4));
        titleFig = horzcat(titleFig,'.pdf');
        %print (fig, '-dpdf', titleFig);
        save2pdf(titleFig);
end

end