% pathLossWedge_beamParam.m: Calculates the path loss at a flat surface, used for
%                 comparing with plane earth loss model, assuming zero
%                 reflection loss. Adjusts the beam pattern according to
%                 the parameters (works only for rx = tx).
% Assumptions: The runway has an upwards triangular shape, zero height does
%              not work.
% Remarks: The heigth differences are assumed to be small (the case for a
%          runway), the signal will therfore have horizontal polarization
%          with respect to the ground.

% Xn: Vector of x-values
% Zn: Vector of corresponding z-values
% deltaX: spacing between the sampling points along the x-axis, if set to 0
%         the spacing is according to Nyquist's theorem.
% tx: Transimtter antenna height
% rx_xArr: Receiver antenna (obseravation points) coordinates along the
%          x-axis.
% rx_zArr: Receiver antenna (observation points) coordinates along the
%          z-axis.
% distance: The distance between the antennas, ignoring height difference
% eField: Calculated electric field at the coordinates to rx_xArr and
%         rx_zArr.
% zs: The height eField is taken from
% frequency: The frequency of operation
% simulationType: Cell-array containing strings with the name of the
%                 simulation types used.


function pathLossWedge_beamParam(Xn,Zn,deltaX,A,tx,rx_xArr,rx_zArr,eField,zs,...
    beta,frequency,simulationType, plotTitle,saveFig)
c = 3*10^8;
lambda = c/frequency;
k = 2*pi/lambda;

tx_x = Xn(1,1); % xVect(1,1);
% rx_x = xVect(numXpoints,1);
tx_z = tx;
%rx_z = rx;

indexMaxZ = find(rx_zArr >= 200 ,1);
rx_xArr = rx_xArr(1:indexMaxZ);
rx_zArr = rx_zArr(1:indexMaxZ);
eField = eField(:,1:indexMaxZ);

[xVect,zVect,eUTDArr] = getUTDfield(Xn,Zn,deltaX,tx_x,tx_z,rx_xArr,rx_zArr,frequency,A);
numXpoints = length(xVect);
rx_zArr = verticalVector(rx_zArr);
numRxPoints = length(rx_zArr);

% Finding the dimensions of the incoming e-field array:
[lines,columns] = size(eField);
if(lines>columns)
    eField = eField';
    numCases = columns;
else
    numCases = lines;
end
cmap = hsv(numCases+1);

plotNames = cell(1+length(simulationType),1);
plotNames{1,1} = 'E-field UTD';
for i = 1: length(simulationType);
    plotNames{i+1,1} = simulationType{i};
end

% Save the figure?
saveFigVal = isempty(saveFig);

switch saveFigVal
    case 1
        % Not save figure
        figure()
    case 0
        % Save figure
        fig = figure('visible','on');
end


%Pl =  (lambda./(4.*pi.*xVect)).*(abs(1 +exp(1j.*k.*2.*tx.*rx./xVect)).^2);
Pl = eUTDArr;
Pl_dB =Pl; % 20.*log10(verticalVector(Pl(1:numRxPoints,1)));
%semilogy(rx_zArr(:,1),Pl_dB,'Color',cmap(1,:));
plot(rx_zArr(:,1),abs(Pl_dB),'Color',cmap(1,:));

xlabel('r[m]');
ylabel('E-field UTD [-dB]');
%legend('Plane Earth Loss');

for i = 1: numCases
    hold on
    
    % Newly inserted:
    pl1 = verticalVector(20.*log10(abs(eField(i,:))));
    pl2 = 0; % - 10.*log10(xVect);
    pathLoss = pl1 + pl2; %20.*log10(abs(eField(i,:))) - 10.*log10(xVect); % - 20.*log10(4.*pi) + 30.*log(lambda);
    semilogx(rx_zArr(:,1),pathLoss(:,1),'Color',cmap(i+1,:)); %,'LineWidth',1*(numCases+1-i));
    % Calculating the path loss for the incoming eField
    %semilogx(xVect,20.*log10(abs(eField(i,:))),colors(i));
    %legend(simulationType(i));
end
legend(plotNames);
title(plotTitle);
grid on

switch saveFigVal
    case 0
        % Save figure
        titleFig = verticalVector(saveFig)';
        regexprep(titleFig,' ','_');
        regexprep(titleFig,'\','_');
        regexprep(titleFig,':','_');
        regexprep(titleFig,'=','_');
        
        if (isempty(strfind(titleFig,'.png')) == 1)
            % File extension needs to be added.
            titleFig = horzcat(titleFig,'.png');
        end
        
        saveas(fig,titleFig,'png');
        print (fig, '-dpdf', titleFig);
end

end













% Calculating the received fields:
% a = tx_z/rx_z;

% Pl = zeros(numXpoints,1);
% for x = 2:numXpoints
%     tx_x = xVect(x);
% % The direct field:
% thetaDir = atan((rx_z-tx_z)/(rx_x-tx_x));
% Adirect = A*exp(-2*log10(2)*((thetaDir/beta)^2));
%
% % The reflected field:
%
% % Find theta:
% d_tx = a.*abs(rx_x-tx_x)./(1+a);
% thetaRefl = (pi/2) - atan(d_tx/tx_z);
% Areflect = A.*exp(-2*log10(2)*((thetaRefl/beta)^2));
%
% Atotal = Adirect + Areflect.*exp(1j.*k.*rx_z.*tx_z./(abs(rx_x-tx_x)));
%
% Pl(x,1) = (lambda./(4.*pi.*(abs(rx_x-tx_x))))*((Atotal/Adirect)^2);
% end

% xVectAux = xVect(2:numXpoints,1);
% thetaDir = atan((rx_z-tx_z)./(abs(tx_x-xVectAux)));
% Adirect = A.*exp(-2.*log10(2).*((thetaDir/beta).^2));
%
% % The reflected field:
%
% % Find theta:
% d_tx = a.*abs(tx_x-xVectAux)./(1+a);
% thetaRefl = (pi/2) - atan(d_tx./tx_z);
% Areflect = A.*exp(-2.*log10(2).*((thetaRefl./beta).^2));
%
% Atotal = Adirect + Areflect.*exp(1j.*k.*rx_z.*tx_z./(abs(tx_x-xVectAux)));
% %Pl = Atotal;
% Pl = (lambda./(4.*pi.*(abs(tx_x-xVectAux)))).*((Atotal./Adirect).^2);


%simulationType = verticalVector(simulationType);
%plotNames = vertcat('Plane Earth Loss', simulationType);
% Calculating the plane earth loss (path loss for plane earth):



