% splitStepAlgorithmAbsorptionLayer.m: Calculates the electric field over a
%                                      flat surface using the split-step
%                                      algorithm. The algorithm inclueds an
%                                      absorption layer at the top.
%               
% Remarks: length(xVect) >= length(zVect)             

% zFieldInit: Vector containing the initial electric field along the z-axis
% zVect: Vector containing the z-coordinates of interest
% xVect: Vector containing the x-coordinates of interest
% maxHeigthInterestZIndex: the z-index that contains the highest index of
%                          interest, above this index there will be 
%                          absorption in order to avoid reflection from the
%                          sky.
% frequency: The frequency of operation
% numPtsAbsoptionLayer: The number of height points in the absorption
%                       layer.

function uValues = splitStepAlgorithmAbsorptionLayer(zFieldInit,zVect, ...
    xVect,maxHeigthInterestZIndex,frequency,numPtsAbsoptionLayer)

c = 3*10^8;
lambda = c/frequency;
k = 2*pi/lambda;
numZpoints = length(zVect);
numXpoints = length(xVect);
Hindex = maxHeigthInterestZIndex;
xVect = verticalVector(xVect);
zVect = verticalVector(zVect);


deltaZvect = zeros(numZpoints,1);
deltaZvect(1:numZpoints-1,1)= zVect(2:numZpoints,1)-zVect(1:numZpoints-1);
deltaZvect(numZpoints,1) = deltaZvect(numZpoints-1,1);

deltaXvect = zeros(numXpoints,1);
deltaXvect(1:numXpoints-1,1)= xVect(2:numXpoints,1)-xVect(1:numXpoints-1);
deltaXvect(numXpoints,1) = deltaXvect(numXpoints-1,1);

uGrid = zeros(numZpoints,numXpoints);
uGrid(:,1)= zFieldInit;
indicesZminusOne = verticalVector(linspace(1,(numZpoints-1),...
    (numZpoints-1)));
Pprime = exp(-1j.*(pi.^2).*(indicesZminusOne.^2)...
    .*deltaXvect(1:1,1)./(2.*k.*(numZpoints.^2)));


Pprime = verticalVector(Pprime);

% Creating the absorption layer:
indexRefraction = createAbsorptionLayer(zFieldInit,...
    Hindex,numPtsAbsoptionLayer);

% Calculating the field:
for x = 2:numXpoints
    
    uGrid(1:(numZpoints-1),x) = inverseDiscreteSineTrans( ...
        Pprime.*discreteSineTrans(uGrid(1:(numZpoints-1),x-1)));
    uGrid(1:numZpoints,x) = exp(1j.*k.*(indexRefraction.^2 -1) ...
        .*deltaXvect(1,1)./2).*uGrid(1:numZpoints,x);
end
uValues = uGrid;

end

